use std::sync::Arc;

use btleplug::platform::{Adapter, Peripheral};
use tauri::{
    async_runtime::{JoinHandle, Mutex, RwLock},
    AppHandle, Runtime,
};

use crate::config_db;

pub struct AppState {
    pub db: Arc<Mutex<sled::Db>>,
    pub central_adapter: Arc<RwLock<Option<Adapter>>>,
    pub central_event_handler: Arc<Mutex<Option<JoinHandle<()>>>>,
    pub scanning: Arc<Mutex<bool>>,
    pub connected_peripheral: Arc<Mutex<Option<Arc<Peripheral>>>>,
}

unsafe impl Send for AppState {}
unsafe impl Sync for AppState {}

impl AppState {
    pub async fn new<R: Runtime>(app: &AppHandle<R>) -> anyhow::Result<Self> {
        let db = config_db::open_config_db(app)?;

        Ok(Self {
            db: Arc::new(Mutex::new(db)),
            central_adapter: Arc::new(RwLock::new(None)),
            central_event_handler: Arc::new(Mutex::new(None)),
            scanning: Arc::new(Mutex::new(false)),
            connected_peripheral: Arc::new(Mutex::new(None)),
        })
    }

    pub async fn set_central_adapter(&self, adapter: Adapter) {
        let mut central_adapter = self.central_adapter.write().await;
        *central_adapter = Some(adapter);
    }

    pub async fn clear_central_adapter(&self) {
        let mut central_adapter = self.central_adapter.write().await;
        *central_adapter = None;
    }

    pub async fn get_central_adapter(&self) -> Option<Adapter> {
        let central_adapter = self.central_adapter.read().await;
        central_adapter.clone()
    }

    pub async fn set_central_event_handler(&self, handler: JoinHandle<()>) {
        let mut central_event_handler = self.central_event_handler.lock().await;
        *central_event_handler = Some(handler);
    }

    pub async fn clear_central_event_handler(&self) {
        let mut central_event_handler = self.central_event_handler.lock().await;
        if let Some(handler) = central_event_handler.take() {
            handler.abort();
        }
    }

    pub async fn set_scanning(&self, scanning_state: bool) {
        let mut scanning = self.scanning.lock().await;
        *scanning = scanning_state;
    }

    pub async fn set_connected_peripheral(&self, peripheral: Peripheral) {
        let mut connected_peripheral = self.connected_peripheral.lock().await;
        *connected_peripheral = Some(Arc::new(peripheral));
    }

    pub async fn clear_connected_peripheral(&self) {
        let mut connected_peripheral = self.connected_peripheral.lock().await;
        *connected_peripheral = None;
    }

    pub async fn get_connected_peripheral(&self) -> Option<Arc<Peripheral>> {
        let connected_peripheral = self.connected_peripheral.lock().await;

        if let Some(peripheral) = connected_peripheral.clone() {
            Some(Arc::clone(&peripheral))
        } else {
            None
        }
    }
}
